window.everit = window.everit || {};
everit.lo = everit.lo || {};
everit.lo.polling = everit.lo.polling || {};

(function(polling, $) {

  var basePath = AJS.contextPath() + "/rest/everit/activity-tracker/latest";
  var currentFlag;

  $(document).ready(function() {
    initPoll();
  });

  function initPoll() {
    var loggedInUser = "";
    loggedInUser = AJS.params.remoteUser;

    if (loggedInUser == null || loggedInUser == "" || loggedInUser == undefined) {
      return; // NOOP if the user is not logged in
    }
    $.get(basePath + "/polling/init", function(jsonData) {
      var data = JSON.parse(jsonData);
      var pollIntervalSeconds = data.pollIntervalSeconds;
      executePollWithTimeout(pollIntervalSeconds);
    }).fail(function(xhr) {
      handleXhrFailure("initPoll()", initPoll, xhr);
    });
  }

  function doPoll() {
    $.get(basePath + "/polling/isRevoked", function(jsonData) {
      var data = JSON.parse(jsonData);
      var accessRevoked = data.accessRevoked;
      if (accessRevoked) {
        closeCurrentFlag();
        currentFlag = AJS.flag({
          type: "warning",
          body: "Your access has been revoked by License Optimizer.<br>" +
                "<div style='text-align: center; margin-top: 2%;'>" +
                "<button class='aui-button aui-button-secondary' onclick='everit.lo.polling.refreshAccess()'>Refresh Access</button>" +
                "</div>",
        });
        return;
      }
      var pollIntervalSeconds = data.pollIntervalSeconds;
      executePollWithTimeout(pollIntervalSeconds);
    }).fail(function(xhr) {
      handleXhrFailure("doPoll()", doPoll, xhr);
    });
  }

  function executePollWithTimeout(pollIntervalSeconds) {
    if (pollIntervalSeconds > 0) { // Zero indicates that polling should be stopped
      setTimeout(doPoll, pollIntervalSeconds * 1000);
    }
  }

  function closeCurrentFlag() {
    if (currentFlag != null && currentFlag != undefined) {
      currentFlag.close();
    }
  }

  function showUnexpectedErrorFlag() {
    closeCurrentFlag();
    currentFlag = AJS.flag({
      type: "error",
      body: "Something bad happened while trying to keep alive your access via the License Optimizer plugin. " + 
            "Please check your browser console and contact your system administrator with the details.",
    });
  }

  function showRefreshFailureFlag() {
    closeCurrentFlag();
    currentFlag = AJS.flag({
      type: "warning",
      body: "The License Optimizer cannot refresh your application access at the moment. Please try again later.<br>" +
            "<div style='text-align: center; margin-top: 2%;'>" +
            "<button class='aui-button aui-button-secondary' onclick='everit.lo.polling.refreshAccess()'>Refresh Access</button>" +
            "</div>",
    });
  }

  function handleXhrFailure(invokedFrom, methodToRetry, xhr) {
    if (xhr.status == 401 || xhr.status == 403) {
      showRefreshFailureFlag();
    } else if (xhr.status == 0 && xhr.statusText == "error") {
      console.warn("[License Optimizer] Failed to invoke " + invokedFrom + " method properly, retry...");
      setTimeout(methodToRetry, 1000);
    } else if (xhr.status == 509) {
      console.error("[License Optimizer] Access polling failed: " + invokedFrom 
                  + " status: " + xhr.status 
                  + " statusText: " + xhr.statusText
                  + " responseText: " + xhr.responseText);
      console.error(xhr);
    } else if (xhr.status == 512) {
      //401 Unauthorized response recieved on the server when calling remote Jira app link, probably Confluence user not exists on the remote Jira.
    } else {
      console.error("[License Optimizer] Access polling failed: " + invokedFrom 
                  + " status: " + xhr.status 
                  + " statusText: " + xhr.statusText);
      console.error(xhr);
      showUnexpectedErrorFlag();
    }
  }

  polling.refreshAccess = function() {
    closeCurrentFlag();
    $.get(basePath + "/access/refresh", function(data, status, xhr) {
      initPoll(); // In case the refresh is successful then start polling again
    }).fail(function(xhr) {
      handleXhrFailure("refreshAccess()", polling.refreshAccess, xhr);
    });
  }

})(everit.lo.polling, jQuery);